using System;
using Server.Network;
using Server.Spells;

namespace Server.Items
{
	public class Spellbook : Item
	{
		public static void Initialize()
		{
			EventSink.OpenSpellbookRequest += new OpenSpellbookRequestEventHandler( EventSink_OpenSpellbookRequest );
			EventSink.CastSpellRequest += new CastSpellRequestEventHandler( EventSink_CastSpellRequest );
		}

		private static void EventSink_OpenSpellbookRequest( OpenSpellbookRequestEventArgs e )
		{
			Mobile from = e.Mobile;

			Spellbook book = from.GetSpellbook( typeof( Spellbook ) ) as Spellbook;

			if ( book != null )
				book.DisplayTo( from );
		}

		private static void EventSink_CastSpellRequest( CastSpellRequestEventArgs e )
		{
			Mobile from = e.Mobile;
			Spellbook book = e.Spellbook as Spellbook;
			int spellID = e.SpellID;

			if ( from.Spell != null )
			{
				from.SendLocalizedMessage( 502642 ); // You are already casting a spell.
			}
			else if ( from.NextSpellTime > DateTime.Now )
			{
				from.SendLocalizedMessage( 502644 );//You must wait for that spell to have an effect.
			}
			else
			{
				if ( book == null )
					book = from.GetSpellbook( typeof( Spellbook ) ) as Spellbook;

				if ( book != null && book.HasSpell( spellID ) )
				{
					Spell spell = SpellRegistry.NewSpell( spellID, from, null );

					if ( spell != null )
						spell.Cast();
					else
						from.SendMessage( "That spell is not currently supported." );
				}
				else
				{
					from.SendLocalizedMessage( 500015 ); // You do not have that spell!
				}
			}
		}

		private ulong m_Content;
		private int m_Count;

		public override bool AllowEquipedCast( Mobile from )
		{
			return true;
		}

		public override bool OnDragDrop( Mobile from, Item dropped )
		{
			if ( dropped is SpellScroll && dropped.Amount == 1 )
			{
				SpellScroll scroll = (SpellScroll)dropped;

				ulong mask = (ulong)1 << scroll.SpellID;

				if ( (m_Content & mask) == 0 )
				{
					m_Content |= mask;
					++m_Count;

					scroll.Delete();

					from.Send( new PlaySound( 0x249, GetWorldLocation() ) );

					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ulong Content
		{
			get
			{
				return m_Content;
			}
			set
			{
				if ( m_Content != value )
				{
					m_Content = value;

					m_Count = 0;

					while ( value > 0 )
					{
						m_Count += (int)(value & 0x1);
						value >>= 1;
					}
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int SpellCount
		{
			get
			{
				return m_Count;
			}
		}

		[Constructable]
		public Spellbook() : base( 0xEFA )
		{
			Weight = 3.0;
			Layer = Layer.OneHanded;
			Newbied = true;
		}

		public bool HasSpell( int spellID )
		{

			return ( spellID >= 0 && spellID < 64 && (m_Content & ((ulong)1 << spellID)) != 0 );
		}

		public Spellbook( Serial serial ) : base( serial )
		{
		}

		public void DisplayTo( Mobile to )
		{
			// The client must know about the spellbook or it will crash!

			if ( Parent == null )
			{
				to.Send( this.WorldPacket );
			}
			else if ( Parent is Item )
			{
				// What will happen if the client doesn't know about our parent?
				to.Send( new ContainerContentUpdate( this ) );
			}
			else if ( Parent is Mobile )
			{
				// What will happen if the client doesn't know about our parent?
				to.Send( new EquipUpdate( this ) );
			}

			to.Send( new DisplaySpellbook( this ) );
			to.Send( new SpellbookContent( m_Count, m_Content, this ) );
		}

		public override void OnSingleClick( Mobile from )
		{
			base.OnSingleClick( from );

			this.LabelTo( from, 1042886, m_Count.ToString() );
		}

		public override void OnDoubleClick( Mobile from )
		{
			DisplayTo( from );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Content );
			writer.Write( m_Count );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Content = reader.ReadULong();
					m_Count = reader.ReadInt();

					break;
				}
			}
		}
	}
}