using System;
using Server.Items;
using Server.Network;
using Server.Targeting;
using Server.Gumps;

namespace Server.Items
{
	public class Bandage : Item, IDyable
	{
		[Constructable]
		public Bandage() : this( 1 )
		{
		}

		[Constructable]
		public Bandage( int amount ) : base( 0xE21 )
		{
			Stackable = true;
			Weight = 0.1;
			Amount = amount;
		}

		public Bandage( Serial serial ) : base( serial )
		{
		}

		public bool Dye( Mobile from, DyeTub sender )
		{
			if ( Deleted ) return false;

			Hue = sender.DyedHue;

			return true;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnDoubleClick( Mobile from )
		{
			from.SendLocalizedMessage( 500948 ); // Who will you use the bandages on?

			from.Target = new InternalTarget( this );
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new Bandage(), amount );
		}

		public void Target( Mobile healer, Mobile target )
		{
			if ( Deleted || !healer.CanSee( this ) ) return;

			int number = -1;

			if ( !healer.CanSee( target ) )
			{
				number = 500237; // Target can not be seen.
			}
			else if ( !target.Poisoned && target.Hits == target.HitsMax )
			{
				number = 500955; // That being is not damaged!
			}
			else
			{
				if ( !healer.BeginAction( typeof( Bandage ) ) )
				{
					number = 500310;// You are busy with something else
				}
				else
				{
				
					bool onSelf = ( healer == target );

					double max = onSelf ? 15.0 : 5.0;
					double sub = onSelf ?  5.0 : 2.0;

					TimeSpan duration = TimeSpan.FromSeconds( max - ((healer.Skills[SkillName.Healing].Value / 100.0) * sub) );

					InternalTimer t = new InternalTimer( this, healer, target, duration );
					t.Start();

					number = 500956; // You begin applying the bandages.
				}
			}

			healer.SendLocalizedMessage( number );
		}

		private class InternalTimer : Timer
		{
			private Mobile m_Healer;
			private Mobile m_Target;
			private Bandage m_Bandage;

			public InternalTimer( Bandage bandage, Mobile healer, Mobile target, TimeSpan duration ) : base( duration )
			{
				m_Bandage = bandage;
				m_Healer = healer;
				m_Target = target;
			}

			protected override void OnTick()
			{
				if ( m_Bandage.Deleted || !m_Healer.CanSee( m_Bandage ) || !m_Healer.CanSee( m_Target ) ) return;

				m_Bandage.Consume();

				int healerNumber=-1, targetNumber=-1;
				bool playSound = true;
				bool checkSkills = false;

				m_Healer.EndAction( typeof( Bandage ) );
				// TODO: Ghost stuff

				if ( !m_Healer.Alive )
				{
					healerNumber = 500962; //You were unable to finish your work before you died.
					targetNumber = -1;
					playSound = false;
				}
				else if ( !m_Healer.InRange( m_Target, 1 ) )
				{
					healerNumber = 500963; // You did not stay close enough to heal your target.
					targetNumber = -1;
					playSound = false;
				}
				else if ( !m_Target.Alive )
				{
					double skill = m_Healer.Skills[SkillName.Healing].Value;
					checkSkills = true;

					if ( skill < Utility.Random( 80, 20 ) )
					{
						healerNumber = 500966; //You are unable to resurrect your patient
					}
					else
					{
						healerNumber = 500965;//You are able to resurrect your patient
						targetNumber = -1;
						m_Target.SendGump( new ResurrectGump( m_Target ) );
					}
				}
				else if ( m_Target.Poisoned )
				{
					m_Target.Poison = null;
					checkSkills = true;

					m_Healer.SendLocalizedMessage( 500969 ); // You finish applying the bandages.

					healerNumber = (m_Healer == m_Target) ? -1 : 1010058; // You have cured the target of all poisons.
					targetNumber = 1010059; // You have been cured of all poisons.
				}
				else if ( m_Target.Hits == m_Target.HitsMax )
				{
					healerNumber = 500967; // You heal what little damage your patient had.
					targetNumber = -1;
				}
				else
				{
					checkSkills = true;
					targetNumber = -1;

					double healing = m_Healer.Skills[SkillName.Healing].Value;
					double anatomy = m_Healer.Skills[SkillName.Anatomy].Value;

					if ( healing < Utility.Random( 76 ) )
					{
						//never fail healing over 75?
						healerNumber = 500968;//You apply the bandages, but they barely help.
						playSound = false;
					}
					else
					{
						healerNumber = 500969; // You finish applying the bandages.

						double min = (anatomy / 5.0) + (healing / 5.0) + 3.0;
						double max = (anatomy / 5.0) + (healing / 2.0) + 10.0;

						double toHeal = min + (Utility.RandomDouble() * (max - min));

						m_Target.Heal( (int) toHeal );
					}
				}

				if ( healerNumber != -1 )
				{
					m_Healer.SendLocalizedMessage( healerNumber );
				}

				if ( targetNumber != -1 )
				{
					m_Target.SendLocalizedMessage( targetNumber );
				}

				if ( playSound )
				{
					m_Target.PlaySound( 0x57 );
				}

				if ( checkSkills )
				{
					m_Healer.CheckSkill( SkillName.Anatomy, 0, 120 );
					m_Healer.CheckSkill( SkillName.Healing, 0, 120 );
				}
			}
		}

		private class InternalTarget : Target
		{
			private Bandage m_Bandage;

			public InternalTarget( Bandage bandage ) : base( 1, false, TargetFlags.Beneficial )
			{
				m_Bandage = bandage;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					m_Bandage.Target( from, (Mobile)targeted );
				}
				else
				{
					from.SendLocalizedMessage( 500970 ); // Bandages can not be used on that.
				}
			}
		}
	}
}