using Server;
using System;
using Server.Items;
using Server.Multis;
using Server.Spells;
using Server.Spells.Sixth;

namespace Server.Regions
{
	public class HouseRegion : Region
	{
		private BaseHouse m_House;

		public HouseRegion( BaseHouse house ) : base( "", "", house.Map )
		{
			Priority = Region.HousePriority;
			LoadFromXml = false;
			m_House = house;
		}

		public override void OnEnter( Mobile m )
		{
			if ( m_House.IsBanned( m ) )
			{
				m.Location = this.GoLocation;
				m.SendLocalizedMessage( 501340 );//You have been banned from this house
				return;
			}

			if ( m_House.IsFriend( m ) )
				m.SendLocalizedMessage( 501293 );//Welcome back to the house, friend.
		}

		public override bool OnDecay( Item item )
		{
			if ( m_House.IsLockedDown( item ) || m_House.IsSecure( item ) )
				return false;
			else
				return base.OnDecay(item);
		}

		public override TimeSpan GetLogoutDelay( Mobile m )
		{
			if ( m_House.IsFriend( m ) )
				return TimeSpan.FromSeconds( 0.5 );
			else
				return base.GetLogoutDelay( m );
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			Mobile from = e.Mobile;

			bool isOwner = ( from.AccessLevel >= AccessLevel.GameMaster || from == m_House.Owner );
			bool isCoOwner = isOwner || m_House.IsCoOwner( from );
			bool isFriend = isCoOwner || m_House.IsFriend( from );

			if ( e.HasKeyword( 0x33 ) ) // remove thyself
			{
				if ( isFriend )
				{
					from.Target = new HouseKickTarget( m_House );
					from.SendLocalizedMessage( 501326 ); // Target the individual to eject from this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x34 ) ) // I ban thee
			{
				if ( isFriend )
				{
					from.Target = new HouseBanTarget( true, m_House );
					from.SendLocalizedMessage( 501325 ); // Target the individual to ban from this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x23 ) ) // I wish to lock this down
			{
				if ( isCoOwner )
				{
					from.Target = new LockdownTarget( false, m_House );
					from.SendLocalizedMessage( 502097 ); // Lock what down?
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x24 ) ) // I wish to release this
			{
				if ( isCoOwner )
				{
					from.Target = new LockdownTarget( true, m_House );
					from.SendLocalizedMessage( 502100 ); // Choose the item you wish to release
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x25 ) ) // I wish to secure this
			{
				if ( isCoOwner )
				{
					from.Target = new  SecureTarget( false, m_House );
					from.SendLocalizedMessage( 502103 ); // Choose the item you wish to secure
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x26 ) ) // I wish to unsecure this
			{
				if ( isCoOwner )
				{
					from.Target = new  SecureTarget( true, m_House );
					from.SendLocalizedMessage( 502106 ); // Choose the item you wish to unsecure
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x27 ) ) // I wish to place a strong box
			{
				if ( isOwner )
				{
					from.SendLocalizedMessage( 502109 ); // Owners do not get a strongbox of their own.
				}
				else if ( isCoOwner )
				{
					m_House.AddStrongBox( from );
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
			else if ( e.HasKeyword( 0x28 ) )
			{
				if ( isCoOwner )
				{
					m_House.AddTrashBarrel( from );
				}
				else if ( isFriend )
				{
					from.SendLocalizedMessage( 1010587 ); // You are not a co-owner of this house.
				}
				else
				{
					from.SendLocalizedMessage( 502094 ); // You must be in your house to do this.
				}
			}
		}

		public override void OnExit( Mobile m )
		{
			//m.SendMessage( "Bye!" );
		}

		public override bool OnBeginSpellCast( Mobile m, Spell s )
		{
			if ( s is MarkSpell )
			{
				m.SendLocalizedMessage( 501800 );//You cannot mark that object at that location
				return false;
			}
			else
				return base.OnBeginSpellCast( m, s );
		}

		public override bool OnDoubleClick( Mobile from, object o )
		{
			if ( o is Container )
			{
				Container i = (Container)o;
				if ( m_House.IsSecure( i ) )
				{
					if ( m_House.IsCoOwner( from ) )
					{
						return true;
					}
					else
					{
						from.SendLocalizedMessage( 501655 );//That is secure!
						return false;
					}
				}
			}
			return true;
		}

		public override bool OnSingleClick( Mobile from, object o )
		{
			if ( o is Item )
			{
				Item i = (Item)o;
				if ( m_House.IsLockedDown( i ) )
				{
					i.LabelTo( from, 501643 );//[locked down]
				} else if ( m_House.IsSecure( i ) )
				{
					i.LabelTo( from, 501644 );//[locked down & secure]
				}
			}
			return true;
		}

		public BaseHouse House
		{
			get
			{
				return m_House;
			}
		}
	}
}